program cstability

  use sep
  use cmodeling_mod

  implicit none
  
  real :: dx, dy, dz, dt, dtt, eps1, eps2, del1, del2, del3, vpz, varphi, the, phi
  real :: d, kxdx, kydy, kzdz
  real :: cx4(2), sx4(2), cy4(2), sy4(2), cz4(2), sz4(2)
  real :: cx8(4), sx8(4), cy8(4), sy8(4), cz8(4), sz8(4)
  real :: cx12(6), sx12(6), cy12(6), sy12(6), cz12(6), sz12(6)
  real :: cx16(8), sx16(8), cy16(8), sy16(8), cz16(8), sz16(8)
  real :: cx2, sx2, cy2, sy2, cz2, sz2
  real :: cx6(3), sx6(3), cy6(3), sy6(3), cz6(3), sz6(3)
  real :: cx10(5), sx10(5), cy10(5), sy10(5), cz10(5), sz10(5)
  real :: cx14(7), sx14(7), cy14(7), sy14(7), cz14(7), sz14(7)
  double precision :: A(3,3), B(6,6), I(3,3), wr(6), wi(6), vs(6,6), work(18), eig(101,101,101)
  integer :: sdim, info, i1, i2, i3, l(8), it
  logical :: bwork(6)

  external select

  call sep_init() 

  d = pi/50.

  call from_param("dx", dx)
  call from_param("dy", dy)
  call from_param("dz", dz)

  call from_param("vpz", vpz)
  call from_param("eps1", eps1)
  call from_param("eps2", eps2)
  call from_param("del1", del1)
  call from_param("del2", del2)
  call from_param("del3", del3)
  call from_param("varphi", varphi)
  call from_param("the", the)
  call from_param("phi", phi)
  
  call init_vel(eps1, eps2, del1, del2, del3, vpz, varphi, the, phi)

  I = 0.
  I(1,1) = 1.
  I(2,2) = 1.
  I(3,3) = 1.

  xx = 1./dx/dx
  xy = 1./dx/dy
  xz = 1./dx/dz
  yy = 1./dy/dy
  yz = 1./dy/dz
  zz = 1./dz/dz

  l = (/ 1, 2, 3, 4 , 5, 6, 7, 8/)

  call init_c4()
  call init_c8()
  call init_c12()
  call init_c16()
  call init_c2()
  call init_c6()
  call init_c10()
  call init_c14()
  call init_b4()
  call init_b8()
  call init_b12()
  call init_b16()
  call init_b2()
  call init_b6()
  call init_b10()
  call init_b14()

  open(4,file="centered_iso_eig4.txt",form="formatted")
  open(8,file="centered_iso_eig8.txt",form="formatted")
  open(12,file="centered_iso_eig12.txt",form="formatted")
  open(16,file="centered_iso_eig16.txt",form="formatted")
  open(2,file="centered_iso_eig2.txt",form="formatted")
  open(6,file="centered_iso_eig6.txt",form="formatted")
  open(10,file="centered_iso_eig10.txt",form="formatted")
  open(14,file="centered_iso_eig14.txt",form="formatted")

  dtt = 0.0001

 do it=1,21
  write(0,*) it

  dt = (it-1)*dtt

  do i3=1,101
    kzdz = -pi + (i3-1)*d
    cz4 = cos(l(1:2)*kzdz)
    sz4 = sin(l(1:2)*kzdz)

    do i2=1,101
      kydy = -pi + (i2-1)*d
      cy4 = cos(l(1:2)*kydy)
      sy4 = sin(l(1:2)*kydy)

      do i1=1,101
        kxdx = -pi + (i1-1)*d
        cx4 = cos(l(1:2)*kxdx)
        sx4 = sin(l(1:2)*kxdx)
        
        A(:,:) = v(:,1,1,:)*xx*2*sum(c4(1:2)*(cx4-1)) + v(:,2,2,:)*yy*2*sum(c4(1:2)*(cy4-1)) &
               + v(:,3,3,:)*zz*2*sum(c4(1:2)*(cz4-1)) &
              - (v(:,1,2,:) + v(:,2,1,:))*xy*4*sum(b4*sx4)*sum(b4*sy4) &
              - (v(:,1,3,:) + v(:,3,1,:))*xz*4*sum(b4*sx4)*sum(b4*sz4) &
              - (v(:,2,3,:) + v(:,3,2,:))*yz*4*sum(b4*sy4)*sum(b4*sz4)

        B = 0.
        B(1:3,1:3) = A*dt*dt + I
        B(1:3,4:6) = -I
        B(4:6,1:3) = I
        B(4:6,4:6) = -I
        
        call dgees('N', 'N', select, 6, B, 6, sdim, wr, wi, vs, 6, work, 18, bwork, info)

        eig(i1,i2,i3) = maxval(sqrt((wr+1)*(wr+1) + wi*wi))

      enddo
    enddo
  enddo
 
  write(4,*) dt, maxval(eig)

 enddo





 do it=1,21
  write(0,*) it

  dt = (it-1)*dtt

  do i3=1,101
    kzdz = -pi + (i3-1)*d
    cz8 = cos(l(1:4)*kzdz)
    sz8 = sin(l(1:4)*kzdz)

    do i2=1,101
      kydy = -pi + (i2-1)*d
      cy8 = cos(l(1:4)*kydy)
      sy8 = sin(l(1:4)*kydy)

      do i1=1,101
        kxdx = -pi + (i1-1)*d
        cx8 = cos(l(1:4)*kxdx)
        sx8 = sin(l(1:4)*kxdx)
        
        A(:,:) = v(:,1,1,:)*xx*2*sum(c8(1:4)*(cx8-1)) + v(:,2,2,:)*yy*2*sum(c8(1:4)*(cy8-1)) &
               + v(:,3,3,:)*zz*2*sum(c8(1:4)*(cz8-1)) &
              - (v(:,1,2,:) + v(:,2,1,:))*xy*4*sum(b8*sx8)*sum(b8*sy8) &
              - (v(:,1,3,:) + v(:,3,1,:))*xz*4*sum(b8*sx8)*sum(b8*sz8) &
              - (v(:,2,3,:) + v(:,3,2,:))*yz*4*sum(b8*sy8)*sum(b8*sz8)

        B = 0.
        B(1:3,1:3) = A*dt*dt + I
        B(1:3,4:6) = -I
        B(4:6,1:3) = I
        B(4:6,4:6) = -I
        
        call dgees('N', 'N', select, 6, B, 6, sdim, wr, wi, vs, 6, work, 18, bwork, info)

        eig(i1,i2,i3) = maxval(sqrt((wr+1)*(wr+1) + wi*wi))

      enddo
    enddo
  enddo
 
  write(8,*) dt, maxval(eig)

 enddo





 do it=1,21
  write(0,*) it

  dt = (it-1)*dtt

  do i3=1,101
    kzdz = -pi + (i3-1)*d
    cz12 = cos(l(1:6)*kzdz)
    sz12 = sin(l(1:6)*kzdz)

    do i2=1,101
      kydy = -pi + (i2-1)*d
      cy12 = cos(l(1:6)*kydy)
      sy12 = sin(l(1:6)*kydy)

      do i1=1,101
        kxdx = -pi + (i1-1)*d
        cx12 = cos(l(1:6)*kxdx)
        sx12 = sin(l(1:6)*kxdx)
        
        A(:,:) = v(:,1,1,:)*xx*2*sum(c12(1:6)*(cx12-1)) + v(:,2,2,:)*yy*2*sum(c12(1:6)*(cy12-1)) &
               + v(:,3,3,:)*zz*2*sum(c12(1:6)*(cz12-1)) &
              - (v(:,1,2,:) + v(:,2,1,:))*xy*4*sum(b12*sx12)*sum(b12*sy12) &
              - (v(:,1,3,:) + v(:,3,1,:))*xz*4*sum(b12*sx12)*sum(b12*sz12) &
              - (v(:,2,3,:) + v(:,3,2,:))*yz*4*sum(b12*sy12)*sum(b12*sz12)

        B = 0.
        B(1:3,1:3) = A*dt*dt + I
        B(1:3,4:6) = -I
        B(4:6,1:3) = I
        B(4:6,4:6) = -I
        
        call dgees('N', 'N', select, 6, B, 6, sdim, wr, wi, vs, 6, work, 18, bwork, info)

        eig(i1,i2,i3) = maxval(sqrt((wr+1)*(wr+1) + wi*wi))

      enddo
    enddo
  enddo
 
  write(12,*) dt, maxval(eig)

 enddo





 do it=1,21
  write(0,*) it

  dt = (it-1)*dtt

  do i3=1,101
    kzdz = -pi + (i3-1)*d
    cz16 = cos(l(1:8)*kzdz)
    sz16 = sin(l(1:8)*kzdz)

    do i2=1,101
      kydy = -pi + (i2-1)*d
      cy16 = cos(l(1:8)*kydy)
      sy16 = sin(l(1:8)*kydy)

      do i1=1,101
        kxdx = -pi + (i1-1)*d
        cx16 = cos(l(1:8)*kxdx)
        sx16 = sin(l(1:8)*kxdx)
        
        A(:,:) = v(:,1,1,:)*xx*2*sum(c16(1:8)*(cx16-1)) + v(:,2,2,:)*yy*2*sum(c16(1:8)*(cy16-1)) &
               + v(:,3,3,:)*zz*2*sum(c16(1:8)*(cz16-1)) &
              - (v(:,1,2,:) + v(:,2,1,:))*xy*4*sum(b16*sx16)*sum(b16*sy16) &
              - (v(:,1,3,:) + v(:,3,1,:))*xz*4*sum(b16*sx16)*sum(b16*sz16) &
              - (v(:,2,3,:) + v(:,3,2,:))*yz*4*sum(b16*sy16)*sum(b16*sz16)

        B = 0.
        B(1:3,1:3) = A*dt*dt + I
        B(1:3,4:6) = -I
        B(4:6,1:3) = I
        B(4:6,4:6) = -I
        
        call dgees('N', 'N', select, 6, B, 6, sdim, wr, wi, vs, 6, work, 18, bwork, info)

        eig(i1,i2,i3) = maxval(sqrt((wr+1)*(wr+1) + wi*wi))

      enddo
    enddo
  enddo
 
  write(16,*) dt, maxval(eig)

 enddo

  
 do it=1,21
  write(0,*) it

  dt = (it-1)*dtt

  do i3=1,101
    kzdz = -pi + (i3-1)*d
    cz2 = cos(kzdz)
    sz2 = sin(kzdz)

    do i2=1,101
      kydy = -pi + (i2-1)*d
      cy2 = cos(kydy)
      sy2 = sin(kydy)

      do i1=1,101
        kxdx = -pi + (i1-1)*d
        cx2 = cos(kxdx)
        sx2 = sin(kxdx)
        
        A(:,:) = v(:,1,1,:)*xx*2*(cx2-1) + v(:,2,2,:)*yy*2*(cy2-1) &
               + v(:,3,3,:)*zz*2*(cz2-1) &
              - (v(:,1,2,:) + v(:,2,1,:))*xy*4*sx2*sy2 &
              - (v(:,1,3,:) + v(:,3,1,:))*xz*4*sx2*sz2 &
              - (v(:,2,3,:) + v(:,3,2,:))*yz*4*sy2*sz2

        B = 0.
        B(1:3,1:3) = A*dt*dt + I
        B(1:3,4:6) = -I
        B(4:6,1:3) = I
        B(4:6,4:6) = -I
        
        call dgees('N', 'N', select, 6, B, 6, sdim, wr, wi, vs, 6, work, 18, bwork, info)

        eig(i1,i2,i3) = maxval(sqrt((wr+1)*(wr+1) + wi*wi))

      enddo
    enddo
  enddo
 
  write(2,*) dt, maxval(eig)

 enddo





 do it=1,21
  write(0,*) it

  dt = (it-1)*dtt

  do i3=1,101
    kzdz = -pi + (i3-1)*d
    cz6 = cos(l(1:3)*kzdz)
    sz6 = sin(l(1:3)*kzdz)

    do i2=1,101
      kydy = -pi + (i2-1)*d
      cy6 = cos(l(1:3)*kydy)
      sy6 = sin(l(1:3)*kydy)

      do i1=1,101
        kxdx = -pi + (i1-1)*d
        cx6 = cos(l(1:3)*kxdx)
        sx6 = sin(l(1:3)*kxdx)
        
        A(:,:) = v(:,1,1,:)*xx*2*sum(c6(1:3)*(cx6-1)) + v(:,2,2,:)*yy*2*sum(c6(1:3)*(cy6-1)) &
               + v(:,3,3,:)*zz*2*sum(c6(1:3)*(cz6-1)) &
              - (v(:,1,2,:) + v(:,2,1,:))*xy*4*sum(b6*sx6)*sum(b6*sy6) &
              - (v(:,1,3,:) + v(:,3,1,:))*xz*4*sum(b6*sx6)*sum(b6*sz6) &
              - (v(:,2,3,:) + v(:,3,2,:))*yz*4*sum(b6*sy6)*sum(b6*sz6)

        B = 0.
        B(1:3,1:3) = A*dt*dt + I
        B(1:3,4:6) = -I
        B(4:6,1:3) = I
        B(4:6,4:6) = -I
        
        call dgees('N', 'N', select, 6, B, 6, sdim, wr, wi, vs, 6, work, 18, bwork, info)

        eig(i1,i2,i3) = maxval(sqrt((wr+1)*(wr+1) + wi*wi))

      enddo
    enddo
  enddo
 
  write(6,*) dt, maxval(eig)

 enddo





 do it=1,21
  write(0,*) it

  dt = (it-1)*dtt

  do i3=1,101
    kzdz = -pi + (i3-1)*d
    cz10 = cos(l(1:5)*kzdz)
    sz10 = sin(l(1:5)*kzdz)

    do i2=1,101
      kydy = -pi + (i2-1)*d
      cy10 = cos(l(1:5)*kydy)
      sy10 = sin(l(1:5)*kydy)

      do i1=1,101
        kxdx = -pi + (i1-1)*d
        cx10 = cos(l(1:5)*kxdx)
        sx10 = sin(l(1:5)*kxdx)
        
        A(:,:) = v(:,1,1,:)*xx*2*sum(c10(1:5)*(cx10-1)) + v(:,2,2,:)*yy*2*sum(c10(1:5)*(cy10-1)) &
               + v(:,3,3,:)*zz*2*sum(c10(1:5)*(cz10-1)) &
              - (v(:,1,2,:) + v(:,2,1,:))*xy*4*sum(b10*sx10)*sum(b10*sy10) &
              - (v(:,1,3,:) + v(:,3,1,:))*xz*4*sum(b10*sx10)*sum(b10*sz10) &
              - (v(:,2,3,:) + v(:,3,2,:))*yz*4*sum(b10*sy10)*sum(b10*sz10)

        B = 0.
        B(1:3,1:3) = A*dt*dt + I
        B(1:3,4:6) = -I
        B(4:6,1:3) = I
        B(4:6,4:6) = -I
        
        call dgees('N', 'N', select, 6, B, 6, sdim, wr, wi, vs, 6, work, 18, bwork, info)

        eig(i1,i2,i3) = maxval(sqrt((wr+1)*(wr+1) + wi*wi))

      enddo
    enddo
  enddo
 
  write(10,*) dt, maxval(eig)

 enddo





 do it=1,21
  write(0,*) it

  dt = (it-1)*dtt

  do i3=1,101
    kzdz = -pi + (i3-1)*d
    cz14 = cos(l(1:7)*kzdz)
    sz14 = sin(l(1:7)*kzdz)

    do i2=1,101
      kydy = -pi + (i2-1)*d
      cy14 = cos(l(1:7)*kydy)
      sy14 = sin(l(1:7)*kydy)

      do i1=1,101
        kxdx = -pi + (i1-1)*d
        cx14 = cos(l(1:7)*kxdx)
        sx14 = sin(l(1:7)*kxdx)
        
        A(:,:) = v(:,1,1,:)*xx*2*sum(c14(1:7)*(cx14-1)) + v(:,2,2,:)*yy*2*sum(c14(1:7)*(cy14-1)) &
               + v(:,3,3,:)*zz*2*sum(c14(1:7)*(cz14-1)) &
              - (v(:,1,2,:) + v(:,2,1,:))*xy*4*sum(b14*sx14)*sum(b14*sy14) &
              - (v(:,1,3,:) + v(:,3,1,:))*xz*4*sum(b14*sx14)*sum(b14*sz14) &
              - (v(:,2,3,:) + v(:,3,2,:))*yz*4*sum(b14*sy14)*sum(b14*sz14)

        B = 0.
        B(1:3,1:3) = A*dt*dt + I
        B(1:3,4:6) = -I
        B(4:6,1:3) = I
        B(4:6,4:6) = -I
        
        call dgees('N', 'N', select, 6, B, 6, sdim, wr, wi, vs, 6, work, 18, bwork, info)

        eig(i1,i2,i3) = maxval(sqrt((wr+1)*(wr+1) + wi*wi))

      enddo
    enddo
  enddo
 
  write(14,*) dt, maxval(eig)

 enddo

  dtt = 0.001

 do it=1,8
  write(0,*) it

  dt = 0.003 + (it-1)*dtt

  do i3=1,101
    kzdz = -pi + (i3-1)*d
    cz4 = cos(l(1:2)*kzdz)
    sz4 = sin(l(1:2)*kzdz)

    do i2=1,101
      kydy = -pi + (i2-1)*d
      cy4 = cos(l(1:2)*kydy)
      sy4 = sin(l(1:2)*kydy)

      do i1=1,101
        kxdx = -pi + (i1-1)*d
        cx4 = cos(l(1:2)*kxdx)
        sx4 = sin(l(1:2)*kxdx)
        
        A(:,:) = v(:,1,1,:)*xx*2*sum(c4(1:2)*(cx4-1)) + v(:,2,2,:)*yy*2*sum(c4(1:2)*(cy4-1)) &
               + v(:,3,3,:)*zz*2*sum(c4(1:2)*(cz4-1)) &
              - (v(:,1,2,:) + v(:,2,1,:))*xy*4*sum(b4*sx4)*sum(b4*sy4) &
              - (v(:,1,3,:) + v(:,3,1,:))*xz*4*sum(b4*sx4)*sum(b4*sz4) &
              - (v(:,2,3,:) + v(:,3,2,:))*yz*4*sum(b4*sy4)*sum(b4*sz4)

        B = 0.
        B(1:3,1:3) = A*dt*dt + I
        B(1:3,4:6) = -I
        B(4:6,1:3) = I
        B(4:6,4:6) = -I
        
        call dgees('N', 'N', select, 6, B, 6, sdim, wr, wi, vs, 6, work, 18, bwork, info)

        eig(i1,i2,i3) = maxval(sqrt((wr+1)*(wr+1) + wi*wi))

      enddo
    enddo
  enddo
 
  write(4,*) dt, maxval(eig)

 enddo





 do it=1,8
  write(0,*) it

  dt = 0.003+(it-1)*dtt

  do i3=1,101
    kzdz = -pi + (i3-1)*d
    cz8 = cos(l(1:4)*kzdz)
    sz8 = sin(l(1:4)*kzdz)

    do i2=1,101
      kydy = -pi + (i2-1)*d
      cy8 = cos(l(1:4)*kydy)
      sy8 = sin(l(1:4)*kydy)

      do i1=1,101
        kxdx = -pi + (i1-1)*d
        cx8 = cos(l(1:4)*kxdx)
        sx8 = sin(l(1:4)*kxdx)
        
        A(:,:) = v(:,1,1,:)*xx*2*sum(c8(1:4)*(cx8-1)) + v(:,2,2,:)*yy*2*sum(c8(1:4)*(cy8-1)) &
               + v(:,3,3,:)*zz*2*sum(c8(1:4)*(cz8-1)) &
              - (v(:,1,2,:) + v(:,2,1,:))*xy*4*sum(b8*sx8)*sum(b8*sy8) &
              - (v(:,1,3,:) + v(:,3,1,:))*xz*4*sum(b8*sx8)*sum(b8*sz8) &
              - (v(:,2,3,:) + v(:,3,2,:))*yz*4*sum(b8*sy8)*sum(b8*sz8)

        B = 0.
        B(1:3,1:3) = A*dt*dt + I
        B(1:3,4:6) = -I
        B(4:6,1:3) = I
        B(4:6,4:6) = -I
        
        call dgees('N', 'N', select, 6, B, 6, sdim, wr, wi, vs, 6, work, 18, bwork, info)

        eig(i1,i2,i3) = maxval(sqrt((wr+1)*(wr+1) + wi*wi))

      enddo
    enddo
  enddo
 
  write(8,*) dt, maxval(eig)

 enddo





 do it=1,8
  write(0,*) it

  dt = 0.003+(it-1)*dtt

  do i3=1,101
    kzdz = -pi + (i3-1)*d
    cz12 = cos(l(1:6)*kzdz)
    sz12 = sin(l(1:6)*kzdz)

    do i2=1,101
      kydy = -pi + (i2-1)*d
      cy12 = cos(l(1:6)*kydy)
      sy12 = sin(l(1:6)*kydy)

      do i1=1,101
        kxdx = -pi + (i1-1)*d
        cx12 = cos(l(1:6)*kxdx)
        sx12 = sin(l(1:6)*kxdx)
        
        A(:,:) = v(:,1,1,:)*xx*2*sum(c12(1:6)*(cx12-1)) + v(:,2,2,:)*yy*2*sum(c12(1:6)*(cy12-1)) &
               + v(:,3,3,:)*zz*2*sum(c12(1:6)*(cz12-1)) &
              - (v(:,1,2,:) + v(:,2,1,:))*xy*4*sum(b12*sx12)*sum(b12*sy12) &
              - (v(:,1,3,:) + v(:,3,1,:))*xz*4*sum(b12*sx12)*sum(b12*sz12) &
              - (v(:,2,3,:) + v(:,3,2,:))*yz*4*sum(b12*sy12)*sum(b12*sz12)

        B = 0.
        B(1:3,1:3) = A*dt*dt + I
        B(1:3,4:6) = -I
        B(4:6,1:3) = I
        B(4:6,4:6) = -I
        
        call dgees('N', 'N', select, 6, B, 6, sdim, wr, wi, vs, 6, work, 18, bwork, info)

        eig(i1,i2,i3) = maxval(sqrt((wr+1)*(wr+1) + wi*wi))

      enddo
    enddo
  enddo
 
  write(12,*) dt, maxval(eig)

 enddo





 do it=1,8
  write(0,*) it

  dt = 0.003+(it-1)*dtt

  do i3=1,101
    kzdz = -pi + (i3-1)*d
    cz16 = cos(l(1:8)*kzdz)
    sz16 = sin(l(1:8)*kzdz)

    do i2=1,101
      kydy = -pi + (i2-1)*d
      cy16 = cos(l(1:8)*kydy)
      sy16 = sin(l(1:8)*kydy)

      do i1=1,101
        kxdx = -pi + (i1-1)*d
        cx16 = cos(l(1:8)*kxdx)
        sx16 = sin(l(1:8)*kxdx)
        
        A(:,:) = v(:,1,1,:)*xx*2*sum(c16(1:8)*(cx16-1)) + v(:,2,2,:)*yy*2*sum(c16(1:8)*(cy16-1)) &
               + v(:,3,3,:)*zz*2*sum(c16(1:8)*(cz16-1)) &
              - (v(:,1,2,:) + v(:,2,1,:))*xy*4*sum(b16*sx16)*sum(b16*sy16) &
              - (v(:,1,3,:) + v(:,3,1,:))*xz*4*sum(b16*sx16)*sum(b16*sz16) &
              - (v(:,2,3,:) + v(:,3,2,:))*yz*4*sum(b16*sy16)*sum(b16*sz16)

        B = 0.
        B(1:3,1:3) = A*dt*dt + I
        B(1:3,4:6) = -I
        B(4:6,1:3) = I
        B(4:6,4:6) = -I
        
        call dgees('N', 'N', select, 6, B, 6, sdim, wr, wi, vs, 6, work, 18, bwork, info)

        eig(i1,i2,i3) = maxval(sqrt((wr+1)*(wr+1) + wi*wi))

      enddo
    enddo
  enddo
 
  write(16,*) dt, maxval(eig)

 enddo

  
 do it=1,8
  write(0,*) it

  dt = 0.003 + (it-1)*dtt

  do i3=1,101
    kzdz = -pi + (i3-1)*d
    cz2 = cos(kzdz)
    sz2 = sin(kzdz)

    do i2=1,101
      kydy = -pi + (i2-1)*d
      cy2 = cos(kydy)
      sy2 = sin(kydy)

      do i1=1,101
        kxdx = -pi + (i1-1)*d
        cx2 = cos(kxdx)
        sx2 = sin(kxdx)
        
        A(:,:) = v(:,1,1,:)*xx*2*(cx2-1) + v(:,2,2,:)*yy*2*(cy2-1) &
               + v(:,3,3,:)*zz*2*(cz2-1) &
              - (v(:,1,2,:) + v(:,2,1,:))*xy*4*sx2*sy2 &
              - (v(:,1,3,:) + v(:,3,1,:))*xz*4*sx2*sz2 &
              - (v(:,2,3,:) + v(:,3,2,:))*yz*4*sy2*sz2

        B = 0.
        B(1:3,1:3) = A*dt*dt + I
        B(1:3,4:6) = -I
        B(4:6,1:3) = I
        B(4:6,4:6) = -I
        
        call dgees('N', 'N', select, 6, B, 6, sdim, wr, wi, vs, 6, work, 18, bwork, info)

        eig(i1,i2,i3) = maxval(sqrt((wr+1)*(wr+1) + wi*wi))

      enddo
    enddo
  enddo
 
  write(2,*) dt, maxval(eig)

 enddo





 do it=1,8
  write(0,*) it

  dt = 0.003+(it-1)*dtt

  do i3=1,101
    kzdz = -pi + (i3-1)*d
    cz6 = cos(l(1:3)*kzdz)
    sz6 = sin(l(1:3)*kzdz)

    do i2=1,101
      kydy = -pi + (i2-1)*d
      cy6 = cos(l(1:3)*kydy)
      sy6 = sin(l(1:3)*kydy)

      do i1=1,101
        kxdx = -pi + (i1-1)*d
        cx6 = cos(l(1:3)*kxdx)
        sx6 = sin(l(1:3)*kxdx)
        
        A(:,:) = v(:,1,1,:)*xx*2*sum(c6(1:3)*(cx6-1)) + v(:,2,2,:)*yy*2*sum(c6(1:3)*(cy6-1)) &
               + v(:,3,3,:)*zz*2*sum(c6(1:3)*(cz6-1)) &
              - (v(:,1,2,:) + v(:,2,1,:))*xy*4*sum(b6*sx6)*sum(b6*sy6) &
              - (v(:,1,3,:) + v(:,3,1,:))*xz*4*sum(b6*sx6)*sum(b6*sz6) &
              - (v(:,2,3,:) + v(:,3,2,:))*yz*4*sum(b6*sy6)*sum(b6*sz6)

        B = 0.
        B(1:3,1:3) = A*dt*dt + I
        B(1:3,4:6) = -I
        B(4:6,1:3) = I
        B(4:6,4:6) = -I
        
        call dgees('N', 'N', select, 6, B, 6, sdim, wr, wi, vs, 6, work, 18, bwork, info)

        eig(i1,i2,i3) = maxval(sqrt((wr+1)*(wr+1) + wi*wi))

      enddo
    enddo
  enddo
 
  write(6,*) dt, maxval(eig)

 enddo





 do it=1,8
  write(0,*) it

  dt = 0.003+(it-1)*dtt

  do i3=1,101
    kzdz = -pi + (i3-1)*d
    cz10 = cos(l(1:5)*kzdz)
    sz10 = sin(l(1:5)*kzdz)

    do i2=1,101
      kydy = -pi + (i2-1)*d
      cy10 = cos(l(1:5)*kydy)
      sy10 = sin(l(1:5)*kydy)

      do i1=1,101
        kxdx = -pi + (i1-1)*d
        cx10 = cos(l(1:5)*kxdx)
        sx10 = sin(l(1:5)*kxdx)
        
        A(:,:) = v(:,1,1,:)*xx*2*sum(c10(1:5)*(cx10-1)) + v(:,2,2,:)*yy*2*sum(c10(1:5)*(cy10-1)) &
               + v(:,3,3,:)*zz*2*sum(c10(1:5)*(cz10-1)) &
              - (v(:,1,2,:) + v(:,2,1,:))*xy*4*sum(b10*sx10)*sum(b10*sy10) &
              - (v(:,1,3,:) + v(:,3,1,:))*xz*4*sum(b10*sx10)*sum(b10*sz10) &
              - (v(:,2,3,:) + v(:,3,2,:))*yz*4*sum(b10*sy10)*sum(b10*sz10)

        B = 0.
        B(1:3,1:3) = A*dt*dt + I
        B(1:3,4:6) = -I
        B(4:6,1:3) = I
        B(4:6,4:6) = -I
        
        call dgees('N', 'N', select, 6, B, 6, sdim, wr, wi, vs, 6, work, 18, bwork, info)

        eig(i1,i2,i3) = maxval(sqrt((wr+1)*(wr+1) + wi*wi))

      enddo
    enddo
  enddo
 
  write(10,*) dt, maxval(eig)

 enddo





 do it=1,8
  write(0,*) it

  dt = 0.003+(it-1)*dtt

  do i3=1,101
    kzdz = -pi + (i3-1)*d
    cz14 = cos(l(1:7)*kzdz)
    sz14 = sin(l(1:7)*kzdz)

    do i2=1,101
      kydy = -pi + (i2-1)*d
      cy14 = cos(l(1:7)*kydy)
      sy14 = sin(l(1:7)*kydy)

      do i1=1,101
        kxdx = -pi + (i1-1)*d
        cx14 = cos(l(1:7)*kxdx)
        sx14 = sin(l(1:7)*kxdx)
        
        A(:,:) = v(:,1,1,:)*xx*2*sum(c14(1:7)*(cx14-1)) + v(:,2,2,:)*yy*2*sum(c14(1:7)*(cy14-1)) &
               + v(:,3,3,:)*zz*2*sum(c14(1:7)*(cz14-1)) &
              - (v(:,1,2,:) + v(:,2,1,:))*xy*4*sum(b14*sx14)*sum(b14*sy14) &
              - (v(:,1,3,:) + v(:,3,1,:))*xz*4*sum(b14*sx14)*sum(b14*sz14) &
              - (v(:,2,3,:) + v(:,3,2,:))*yz*4*sum(b14*sy14)*sum(b14*sz14)

        B = 0.
        B(1:3,1:3) = A*dt*dt + I
        B(1:3,4:6) = -I
        B(4:6,1:3) = I
        B(4:6,4:6) = -I
        
        call dgees('N', 'N', select, 6, B, 6, sdim, wr, wi, vs, 6, work, 18, bwork, info)

        eig(i1,i2,i3) = maxval(sqrt((wr+1)*(wr+1) + wi*wi))

      enddo
    enddo
  enddo
 
  write(14,*) dt, maxval(eig)

 enddo

  close(8)
  close(12)
  close(4)
  close(16)
  close(2)
  close(10)
  close(6)
  close(14)

  call sep_close()

end program 

logical function select(u,v)

  implicit none

  double precision :: u, v

  select = .true.

end function
