module smodeling_mod

  implicit none
  
  real, private :: dx, dy, dz, dt
  real :: v(3,3,3,3), xx, xy, xz, yy, yz, zz
  real, parameter :: pi = 3.14159265359
  integer, private :: nx, ny, nz, nt
  real :: c2(0:1), c4(0:2), c6(0:3), c8(0:4), c10(0:5), c12(0:6), c14(0:7), c16(0:8)
  real :: b2, b4(2), b6(3), b8(4), b10(5), b12(6), b14(7), b16(8)
  real :: s2, s4(2), s6(3), s8(4), s10(5), s12(6), s14(7), s16(8)

  contains 

  subroutine init_c16()
    ! Calculate coefficients of 16th order finite difference
    	  
    c16(1) = 1.
    c16(2) = -7./40.
    c16(3) = 7./165.
    c16(4) = -7./704.
    c16(5) = 7./3575.
    c16(6) = -1./3432.
    c16(7) = 1./35035.
    c16(8) = -1./732160.
    c16(0) = -2*sum(c16(1:8))
    c16 = 16./9.*c16
	  
  end subroutine

  subroutine init_c14()
    ! Calculate coefficients of 14th order finite difference

    c14(1) = 1.
    c14(2) = -1./6.
    c14(3) = 1./27.
    c14(4) = -1./132.
    c14(5) = 1./825.
    c14(6) = -1./7722.
    c14(7) = -1./147147.
    c14(0) = -2*sum(c14(1:7))
    c14 = 7./4.*c14
	  
  end subroutine

  subroutine init_c12()
    ! Calculate coefficients of 12th order finite difference

    c12(1) = 1.
    c12(2) = -5./32.
    c12(3) = 5./162.
    c12(4) = -1./192.
    c12(5) = 1./1650.
    c12(6) = -1./28512.
    c12(0) = -2*sum(c12(1:6))
    c12 = 12./7.*c12
	  
  end subroutine

  subroutine init_c10()
    ! Calculate coefficients of 10th order finite difference

    c10(1) = 1.
    c10(2) = -1./7.
    c10(3) = 1./42.
    c10(4) = -1./336.
    c10(5) = 1./5250.
    c10(0) = -2*sum(c10(1:5))
    c10 = 5./3.*c10
	  
  end subroutine

  subroutine init_c8()
    ! Calculate coefficients of 8th order finite difference

    c8(1) = 1.
    c8(2) = -1./8.
    c8(3) = 1./63.
    c8(4) = -1./896.
    c8(0) = -2*sum(c8(1:4))
    c8 = 8./5.*c8
	  
  end subroutine

  subroutine init_c6()
    ! Calculate coefficients of 6th order finite difference

    c6(1) = 1.
    c6(2) = -1./10.
    c6(3) = 1./135.
    c6(0) = -2*sum(c6(1:3))
    c6 = 3./2.*c6
	  
  end subroutine

  subroutine init_c4()
    ! Calculate coefficients of 4th order finite difference

    c4(1) = 1.
    c4(2) = -1./16.
    c4(0) = -2*sum(c4(1:2))
    c4 = 4./3.*c4
	  
  end subroutine

  subroutine init_c2()
    ! Calculate coefficients of 2th order finite difference

    c2(1) = 1.
    c2(0) = -2
	  
  end subroutine

  subroutine init_s16()
    ! Calculate coefficients of 16th order finite difference
    	  
    s16(1) = 1.
    s16(2) = -7./81.
    s16(3) = 7./375.
    s16(4) = -1./231.
    s16(5) = 7./8019.
    s16(6) = -7./51909.
    s16(7) = 1./72501.
    s16(8) = -1./1447875.
    s16 = 41409225./33554432.*s16
	  
  end subroutine

  subroutine init_s14()
    ! Calculate coefficients of 14th order finite difference

    s14(1) = 1.
    s14(2) = -1./12.
    s14(3) = 1./60.
    s14(4) = -1./294.
    s14(5) = 1./1782.
    s14(6) = -1./15972.
    s14(7) = -1./290004.
    s14 = 1288287./1048576.*s14
	  
  end subroutine

  subroutine init_s12()
    ! Calculate coefficients of 12th order finite difference

    s12(1) = 1.
    s12(2) = -5./63.
    s12(3) = 1./70.
    s12(4) = -5./2058.
    s12(5) = 1./3402.
    s12(6) = -1./55902.
    s12 = 160083./131072.*s12
	  
  end subroutine

  subroutine init_s10()
    ! Calculate coefficients of 10th order finite difference

    s10(1) = 1.
    s10(2) = -2./27.
    s10(3) = 2./175.
    s10(4) = -1./686.
    s10(5) = 1./10206.
    s10 = 19845./16384.*s10
	  
  end subroutine

  subroutine init_s8()
    ! Calculate coefficients of 8th order finite difference

    s8(1) = 1.
    s8(2) = -1./15.
    s8(3) = 1./125.
    s8(4) = -1./1715.
    s8 = 1225./1024.*s8
	  
  end subroutine

  subroutine init_s6()
    ! Calculate coefficients of 6th order finite difference

    s6(1) = 1.
    s6(2) = -1./18.
    s6(3) = 1./250.
    s6 = 75./64.*s6
	  
  end subroutine

  subroutine init_s4()
    ! Calculate coefficients of 4th order finite difference

    s4(1) = 1.
    s4(2) = -1./27.
    s4 = 9./8.*s4
	  
  end subroutine

  subroutine init_s2()
    ! Calculate coefficients of 2th order finite difference

    s2 = 1.
	  
  end subroutine

  subroutine init_b16()
    ! Calculate coefficients of 16th order finite difference
    	  
    b16(1) = 1.
    b16(2) = -7./20.
    b16(3) = 7./55.
    b16(4) = -7./176.
    b16(5) = 7./715.
    b16(6) = -1./572.
    b16(7) = 1./5005.
    b16(8) = -1./91520.
    b16 = 8./9.*b16
	  
  end subroutine

  subroutine init_b14()
    ! Calculate coefficients of 14th order finite difference

    b14(1) = 1.
    b14(2) = -1./3.
    b14(3) = 1./9.
    b14(4) = -1./33.
    b14(5) = 1./165.
    b14(6) = -1./1287.
    b14(7) = -1./21021.
    b14 = 7./8.*b14
	  
  end subroutine

  subroutine init_b12()
    ! Calculate coefficients of 12th order finite difference

    b12(1) = 1.
    b12(2) = -5./16.
    b12(3) = 5./54.
    b12(4) = -1./48.
    b12(5) = 1./330.
    b12(6) = -1./4752.
    b12 = 6./7.*b12
	  
  end subroutine

  subroutine init_b10()
    ! Calculate coefficients of 10th order finite difference

    b10(1) = 1.
    b10(2) = -2./7.
    b10(3) = 1./14.
    b10(4) = -1./84.
    b10(5) = 1./1050.
    b10 = 5./6.*b10
	  
  end subroutine

  subroutine init_b8()
    ! Calculate coefficients of 8th order finite difference

    b8(1) = 1.
    b8(2) = -1./4.
    b8(3) = 1./21.
    b8(4) = -1./224.
    b8 = 4./5.*b8
	  
  end subroutine

  subroutine init_b6()
    ! Calculate coefficients of 6th order finite difference

    b6(1) = 1.
    b6(2) = -1./5.
    b6(3) = 1./45.
    b6 = 3./4.*b6
	  
  end subroutine

  subroutine init_b4()
    ! Calculate coefficients of 4th order finite difference

    b4(1) = 1.
    b4(2) = -1./8.
    b4 = 2./3.*b4
	  
  end subroutine

  subroutine init_b2()
    ! Calculate coefficients of 2th order finite difference

    b2 = 1.
	  
  end subroutine
  
  subroutine init(dx_in, dy_in, dz_in, dt_in, nx_in, ny_in, nz_in, nt_in)
  ! Initialize grid dimensions, time and spatial steps for the module

    real :: dx_in, dy_in, dz_in, dt_in
    integer :: nx_in, ny_in, nz_in, nt_in
	
      dx = dx_in
      dy = dy_in
      dz = dz_in
      dt = dt_in
      nx = nx_in
      ny = ny_in
      nz = nz_in
      nt = nt_in

    xx = 1./dx/dx
    xy = 1./dx/dy
    xz = 1./dx/dz
    yy = 1./dy/dy
    yz = 1./dy/dz
    zz = 1./dz/dz
	
  end subroutine  
  
  subroutine init_vel(eps1, eps2, del1, del2, del3, vpz, varphi, the, phi)
  
    real, intent(in) :: eps1, eps2, del1, del2, del3, vpz, varphi, the, phi
    real :: G(3,3), R(3,3)
    integer :: i, k, l, n, j, m

    ! Calculate elastic moduli normalized by density already
    G(3,3) = vpz*vpz
    G(1,1) = G(3,3)*(1+2*eps2)
    G(1,2) = G(1,1)*sqrt(1+2*del3)
    G(2,1) = G(1,2)
    G(1,3) = G(3,3)*sqrt(1+2*del2)
    G(3,1) = G(1,3)
    G(2,2) = G(3,3)*(1+2*eps1)
    G(2,3) = G(3,3)*sqrt(1+2*del1)
    G(3,2) = G(2,3)

    ! Calculate the rotationl matrix
    R(1,1) = cos(varphi)*cos(the)*cos(phi) - sin(varphi)*sin(phi)
    R(1,2) = cos(varphi)*cos(the)*sin(phi) + sin(varphi)*cos(phi)
    R(1,3) = -cos(varphi)*sin(the)
    R(2,1) = -sin(varphi)*cos(the)*cos(phi) - cos(varphi)*sin(phi)
    R(2,2) = -sin(varphi)*cos(the)*sin(phi) + cos(varphi)*cos(phi)
    R(2,3) = sin(varphi)*sin(the)
    R(3,1) = sin(the)*cos(phi)
    R(3,2) = sin(the)*sin(phi)
    R(3,3) = cos(the)
	
    ! Calculate "velocity squared"
    v = 0.
    do k=1,3
      do l=1,3
	do n=1,3
          do i=1,3
	    do j=1,3
	      do m=1,3
	        v(i,n,l,k) = v(i,n,l,k) + G(i,j)*R(j,m)*R(j,n)*R(k,m)*R(k,l)
	      enddo
	    enddo
	  enddo
	enddo
      enddo
    enddo
	
  end subroutine

  subroutine one_step8(prev, cur, next)
    ! Calculate the next wave field from the current and previous fields

    real :: prev(nx,ny,nz,3), cur(nx,ny,nz,3), next(nx,ny,nz,3)
    real :: u(2:3,nx-1,ny-1,nz-1,3), tempcur(nx+6,ny+6,nz+6,3)
    real :: tempd(4,3), temp(3), tempdu(2:3,4,3)
    integer :: ix, iy, iz, n, l, k, iux, iuy, iuz

    next = 0.
    u = 0.

    !Pad 0 to cur
    tempcur = 0.
    tempcur(4:nx+3,4:ny+3,4:nz+3,:) = cur

    !$omp parallel do default(shared) private(ix,iy,iz,iux,iuy,iuz,tempd)
    do iuz=1,nz-1 ! Loop over spatial domain ignoring the boundary
      iz = iuz + 3
      do iuy=1,ny-1
        iy = iuy + 3
        do iux=1,nx-1
          ix = iux + 3

          ! d/dd1 
          tempd(1,:) = s8(1)*(tempcur(ix+1,iy+1,iz+1,:) - tempcur(ix  ,iy  ,iz  ,:)) &
                     + s8(2)*(tempcur(ix+2,iy+2,iz+2,:) - tempcur(ix-1,iy-1,iz-1,:)) &
                     + s8(3)*(tempcur(ix+3,iy+3,iz+3,:) - tempcur(ix-2,iy-2,iz-2,:)) &
                     + s8(4)*(tempcur(ix+4,iy+4,iz+4,:) - tempcur(ix-3,iy-3,iz-3,:))

          ! d/dd2 
          tempd(2,:) = s8(1)*(tempcur(ix+1,iy+1,iz  ,:) - tempcur(ix  ,iy  ,iz+1,:)) &
                     + s8(2)*(tempcur(ix+2,iy+2,iz-1,:) - tempcur(ix-1,iy-1,iz+2,:)) &
                     + s8(3)*(tempcur(ix+3,iy+3,iz-2,:) - tempcur(ix-2,iy-2,iz+3,:)) &
                     + s8(4)*(tempcur(ix+4,iy+4,iz-3,:) - tempcur(ix-3,iy-3,iz+4,:))

          ! d/dd3 
          tempd(3,:) = s8(1)*(tempcur(ix+1,iy  ,iz+1,:) - tempcur(ix  ,iy+1,iz  ,:)) &
                     + s8(2)*(tempcur(ix+2,iy-1,iz+2,:) - tempcur(ix-1,iy+2,iz-1,:)) &
                     + s8(3)*(tempcur(ix+3,iy-2,iz+3,:) - tempcur(ix-2,iy+3,iz-2,:)) &
                     + s8(4)*(tempcur(ix+4,iy-3,iz+4,:) - tempcur(ix-3,iy+4,iz-3,:))

          ! d/dd4 
          tempd(4,:) = s8(1)*(tempcur(ix+1,iy  ,iz  ,:) - tempcur(ix  ,iy+1,iz+1,:)) &
                     + s8(2)*(tempcur(ix+2,iy-1,iz-1,:) - tempcur(ix-1,iy+2,iz+2,:)) &
                     + s8(3)*(tempcur(ix+3,iy-2,iz-2,:) - tempcur(ix-2,iy+3,iz+3,:)) &
                     + s8(4)*(tempcur(ix+4,iy-3,iz-3,:) - tempcur(ix-3,iy+4,iz+4,:))
        
          ! d/dx2
          u(2,iux,iuy,iuz,:) = 0.25/dy*(tempd(1,:)+tempd(2,:)-tempd(3,:)-tempd(4,:))

          ! d/dx3
          u(3,iux,iuy,iuz,:) = 0.25/dz*(tempd(1,:)-tempd(2,:)+tempd(3,:)-tempd(4,:))

        enddo
      enddo
    enddo
    !$omp end parallel do

    !$omp parallel do default(shared) private(ix,iy,iz,iux,iuy,iuz,temp,tempdu)
    do iz=5,nz-4 ! Loop over spatial domain ignoring the boundary
      iuz = iz-1
      do iy=5,ny-4
        iuy = iy-1
        do ix=5,nx-4
          iux = ix-1
  
          ! d^2/dx1^2 
          temp = xx*(c8(0)*cur(ix,iy,iz,:) + c8(1)*(cur(ix+1,iy,iz,:) + cur(ix-1,iy,iz,:)) &
                                           + c8(2)*(cur(ix+2,iy,iz,:) + cur(ix-2,iy,iz,:)) &
                                           + c8(3)*(cur(ix+3,iy,iz,:) + cur(ix-3,iy,iz,:)) &
                                           + c8(4)*(cur(ix+4,iy,iz,:) + cur(ix-4,iy,iz,:)))
          next(ix,iy,iz,:) = matmul(v(:,1,1,:),temp)

          ! d^2/dx2^2 
          temp = yy*(c8(0)*cur(ix,iy,iz,:) + c8(1)*(cur(ix,iy+1,iz,:) + cur(ix,iy-1,iz,:)) &
                                           + c8(2)*(cur(ix,iy+2,iz,:) + cur(ix,iy-2,iz,:)) &
                                           + c8(3)*(cur(ix,iy+3,iz,:) + cur(ix,iy-3,iz,:)) &
                                           + c8(4)*(cur(ix,iy+4,iz,:) + cur(ix,iy-4,iz,:)))
          next(ix,iy,iz,:) = next(ix,iy,iz,:) + matmul(v(:,2,2,:),temp)

          ! d^2/dx3^2 
          temp = zz*(c8(0)*cur(ix,iy,iz,:) + c8(1)*(cur(ix,iy,iz+1,:) + cur(ix,iy,iz-1,:)) &
                                           + c8(2)*(cur(ix,iy,iz+2,:) + cur(ix,iy,iz-2,:)) &
                                           + c8(3)*(cur(ix,iy,iz+3,:) + cur(ix,iy,iz-3,:)) &
                                           + c8(4)*(cur(ix,iy,iz+4,:) + cur(ix,iy,iz-4,:)))
          next(ix,iy,iz,:) = next(ix,iy,iz,:) + matmul(v(:,3,3,:),temp)

          ! du/dd1 
          tempdu(:,1,:) = s8(1)*(u(:,iux+1,iuy+1,iuz+1,:) - u(:,iux  ,iuy  ,iuz  ,:)) &
                        + s8(2)*(u(:,iux+2,iuy+2,iuz+2,:) - u(:,iux-1,iuy-1,iuz-1,:)) &
                        + s8(3)*(u(:,iux+3,iuy+3,iuz+3,:) - u(:,iux-2,iuy-2,iuz-2,:)) &
                        + s8(4)*(u(:,iux+4,iuy+4,iuz+4,:) - u(:,iux-3,iuy-3,iuz-3,:))

          ! du/dd2 
          tempdu(:,2,:) = s8(1)*(u(:,iux+1,iuy+1,iuz  ,:) - u(:,iux  ,iuy  ,iuz+1,:)) &
                        + s8(2)*(u(:,iux+2,iuy+2,iuz-1,:) - u(:,iux-1,iuy-1,iuz+2,:)) &
                        + s8(3)*(u(:,iux+3,iuy+3,iuz-2,:) - u(:,iux-2,iuy-2,iuz+3,:)) &
                        + s8(4)*(u(:,iux+4,iuy+4,iuz-3,:) - u(:,iux-3,iuy-3,iuz+4,:))

          ! du/dd3 
          tempdu(:,3,:) = s8(1)*(u(:,iux+1,iuy  ,iuz+1,:) - u(:,iux  ,iuy+1,iuz  ,:)) &
                        + s8(2)*(u(:,iux+2,iuy-1,iuz+2,:) - u(:,iux-1,iuy+2,iuz-1,:)) &
                        + s8(3)*(u(:,iux+3,iuy-2,iuz+3,:) - u(:,iux-2,iuy+3,iuz-2,:)) &
                        + s8(4)*(u(:,iux+4,iuy-3,iuz+4,:) - u(:,iux-3,iuy+4,iuz-3,:))

          ! du/dd4 
          tempdu(:,4,:) = s8(1)*(u(:,iux+1,iuy  ,iuz  ,:) - u(:,iux  ,iuy+1,iuz+1,:)) &
                        + s8(2)*(u(:,iux+2,iuy-1,iuz-1,:) - u(:,iux-1,iuy+2,iuz+2,:)) &
                        + s8(3)*(u(:,iux+3,iuy-2,iuz-2,:) - u(:,iux-2,iuy+3,iuz+3,:)) &
                        + s8(4)*(u(:,iux+4,iuy-3,iuz-3,:) - u(:,iux-3,iuy+4,iuz+4,:))
        
          ! du/dx1
          temp = 0.25/dx*(tempdu(2,1,:)+tempdu(2,2,:)+tempdu(2,3,:)+tempdu(2,4,:))

          ! d^2/dx1dx2 
          next(ix,iy,iz,:) = next(ix,iy,iz,:) + matmul(v(:,1,2,:)+v(:,2,1,:),temp)  

          ! du/dx1
          temp = 0.25/dx*(tempdu(3,1,:)+tempdu(3,2,:)+tempdu(3,3,:)+tempdu(3,4,:))

          ! d^2/dx1dx3 
          next(ix,iy,iz,:) = next(ix,iy,iz,:) + matmul(v(:,1,3,:)+v(:,3,1,:),temp) 

          ! du/dx2
          temp = 0.25/dy*(tempdu(3,1,:)+tempdu(3,2,:)-tempdu(3,3,:)-tempdu(3,4,:))

          ! d^2/dx2dx3 
          next(ix,iy,iz,:) = next(ix,iy,iz,:) + matmul(v(:,3,2,:)+v(:,2,3,:),temp) 
        
          ! Update the next wave field
          next(ix,iy,iz,:) = dt*dt*next(ix,iy,iz,:) - prev(ix,iy,iz,:) + 2*cur(ix,iy,iz,:)

        enddo
      enddo
    enddo
    !$omp end parallel do

  end subroutine

  subroutine one_step6(prev, cur, next)
    ! Calculate the next wave field from the current and previous fields

    real :: prev(nx,ny,nz,3), cur(nx,ny,nz,3), next(nx,ny,nz,3)
    real :: u(2:3,nx-1,ny-1,nz-1,3), tempcur(nx+4,ny+4,nz+4,3)
    real :: tempd(4,3), temp(3), tempdu(2:3,4,3)
    integer :: ix, iy, iz, n, l, k, iux, iuy, iuz

    next = 0.
    u = 0.

    !Pad 0 to cur
    tempcur = 0.
    tempcur(3:nx+2,3:ny+2,3:nz+2,:) = cur

    !$omp parallel do default(shared) private(ix,iy,iz,iux,iuy,iuz,tempd)
    do iuz=1,nz-1 ! Loop over spatial domain ignoring the boundary
      iz = iuz + 2
      do iuy=1,ny-1
        iy = iuy + 2
        do iux=1,nx-1
          ix = iux + 2

          ! d/dd1 
          tempd(1,:) = s6(1)*(tempcur(ix+1,iy+1,iz+1,:) - tempcur(ix  ,iy  ,iz  ,:)) &
                     + s6(2)*(tempcur(ix+2,iy+2,iz+2,:) - tempcur(ix-1,iy-1,iz-1,:)) &
                     + s6(3)*(tempcur(ix+3,iy+3,iz+3,:) - tempcur(ix-2,iy-2,iz-2,:)) 
                     
          ! d/dd2 
          tempd(2,:) = s6(1)*(tempcur(ix+1,iy+1,iz  ,:) - tempcur(ix  ,iy  ,iz+1,:)) &
                     + s6(2)*(tempcur(ix+2,iy+2,iz-1,:) - tempcur(ix-1,iy-1,iz+2,:)) &
                     + s6(3)*(tempcur(ix+3,iy+3,iz-2,:) - tempcur(ix-2,iy-2,iz+3,:)) 

          ! d/dd3 
          tempd(3,:) = s6(1)*(tempcur(ix+1,iy  ,iz+1,:) - tempcur(ix  ,iy+1,iz  ,:)) &
                     + s6(2)*(tempcur(ix+2,iy-1,iz+2,:) - tempcur(ix-1,iy+2,iz-1,:)) &
                     + s6(3)*(tempcur(ix+3,iy-2,iz+3,:) - tempcur(ix-2,iy+3,iz-2,:)) 

          ! d/dd4 
          tempd(4,:) = s6(1)*(tempcur(ix+1,iy  ,iz  ,:) - tempcur(ix  ,iy+1,iz+1,:)) &
                     + s6(2)*(tempcur(ix+2,iy-1,iz-1,:) - tempcur(ix-1,iy+2,iz+2,:)) &
                     + s6(3)*(tempcur(ix+3,iy-2,iz-2,:) - tempcur(ix-2,iy+3,iz+3,:)) 
        
          ! d/dx2
          u(2,iux,iuy,iuz,:) = 0.25/dy*(tempd(1,:)+tempd(2,:)-tempd(3,:)-tempd(4,:))

          ! d/dx3
          u(3,iux,iuy,iuz,:) = 0.25/dz*(tempd(1,:)-tempd(2,:)+tempd(3,:)-tempd(4,:))

        enddo
      enddo
    enddo
    !$omp end parallel do

    !$omp parallel do default(shared) private(ix,iy,iz,iux,iuy,iuz,temp,tempdu)
    do iz=4,nz-3 ! Loop over spatial domain ignoring the boundary
      iuz = iz-1
      do iy=4,ny-3
        iuy = iy-1
        do ix=4,nx-3
          iux = ix-1
  
          ! d^2/dx1^2 
          temp = xx*(c6(0)*cur(ix,iy,iz,:) + c6(1)*(cur(ix+1,iy,iz,:) + cur(ix-1,iy,iz,:)) &
                                           + c6(2)*(cur(ix+2,iy,iz,:) + cur(ix-2,iy,iz,:)) &
                                           + c6(3)*(cur(ix+3,iy,iz,:) + cur(ix-3,iy,iz,:))) 
          next(ix,iy,iz,:) = matmul(v(:,1,1,:),temp)

          ! d^2/dx2^2 
          temp = yy*(c6(0)*cur(ix,iy,iz,:) + c6(1)*(cur(ix,iy+1,iz,:) + cur(ix,iy-1,iz,:)) &
                                           + c6(2)*(cur(ix,iy+2,iz,:) + cur(ix,iy-2,iz,:)) &
                                           + c6(3)*(cur(ix,iy+3,iz,:) + cur(ix,iy-3,iz,:))) 
          next(ix,iy,iz,:) = next(ix,iy,iz,:) + matmul(v(:,2,2,:),temp)

          ! d^2/dx3^2 
          temp = zz*(c6(0)*cur(ix,iy,iz,:) + c6(1)*(cur(ix,iy,iz+1,:) + cur(ix,iy,iz-1,:)) &
                                           + c6(2)*(cur(ix,iy,iz+2,:) + cur(ix,iy,iz-2,:)) &
                                           + c6(3)*(cur(ix,iy,iz+3,:) + cur(ix,iy,iz-3,:))) 
          next(ix,iy,iz,:) = next(ix,iy,iz,:) + matmul(v(:,3,3,:),temp)

          ! du/dd1 
          tempdu(:,1,:) = s6(1)*(u(:,iux+1,iuy+1,iuz+1,:) - u(:,iux  ,iuy  ,iuz  ,:)) &
                        + s6(2)*(u(:,iux+2,iuy+2,iuz+2,:) - u(:,iux-1,iuy-1,iuz-1,:)) &
                        + s6(3)*(u(:,iux+3,iuy+3,iuz+3,:) - u(:,iux-2,iuy-2,iuz-2,:)) 

          ! du/dd2 
          tempdu(:,2,:) = s6(1)*(u(:,iux+1,iuy+1,iuz  ,:) - u(:,iux  ,iuy  ,iuz+1,:)) &
                        + s6(2)*(u(:,iux+2,iuy+2,iuz-1,:) - u(:,iux-1,iuy-1,iuz+2,:)) &
                        + s6(3)*(u(:,iux+3,iuy+3,iuz-2,:) - u(:,iux-2,iuy-2,iuz+3,:)) 

          ! du/dd3 
          tempdu(:,3,:) = s6(1)*(u(:,iux+1,iuy  ,iuz+1,:) - u(:,iux  ,iuy+1,iuz  ,:)) &
                        + s6(2)*(u(:,iux+2,iuy-1,iuz+2,:) - u(:,iux-1,iuy+2,iuz-1,:)) &
                        + s6(3)*(u(:,iux+3,iuy-2,iuz+3,:) - u(:,iux-2,iuy+3,iuz-2,:)) 

          ! du/dd4 
          tempdu(:,4,:) = s6(1)*(u(:,iux+1,iuy  ,iuz  ,:) - u(:,iux  ,iuy+1,iuz+1,:)) &
                        + s6(2)*(u(:,iux+2,iuy-1,iuz-1,:) - u(:,iux-1,iuy+2,iuz+2,:)) &
                        + s6(3)*(u(:,iux+3,iuy-2,iuz-2,:) - u(:,iux-2,iuy+3,iuz+3,:)) 
        
          ! du/dx1
          temp = 0.25/dx*(tempdu(2,1,:)+tempdu(2,2,:)+tempdu(2,3,:)+tempdu(2,4,:))

          ! d^2/dx1dx2 
          next(ix,iy,iz,:) = next(ix,iy,iz,:) + matmul(v(:,1,2,:)+v(:,2,1,:),temp)  

          ! du/dx1
          temp = 0.25/dx*(tempdu(3,1,:)+tempdu(3,2,:)+tempdu(3,3,:)+tempdu(3,4,:))

          ! d^2/dx1dx3 
          next(ix,iy,iz,:) = next(ix,iy,iz,:) + matmul(v(:,1,3,:)+v(:,3,1,:),temp) 

          ! du/dx2
          temp = 0.25/dy*(tempdu(3,1,:)+tempdu(3,2,:)-tempdu(3,3,:)-tempdu(3,4,:))

          ! d^2/dx2dx3 
          next(ix,iy,iz,:) = next(ix,iy,iz,:) + matmul(v(:,3,2,:)+v(:,2,3,:),temp) 
        
          ! Update the next wave field
          next(ix,iy,iz,:) = dt*dt*next(ix,iy,iz,:) - prev(ix,iy,iz,:) + 2*cur(ix,iy,iz,:)

        enddo
      enddo
    enddo
    !$omp end parallel do

  end subroutine

end module		 
